########################################################################################################################
### Chromosome Sorting

## By default the sorting of samtools faidx is not good, but we can resort it, and samtools sort will respect this sorting order
## To fix existing alginments either use ReorderSam or reheader | resort with samtools

## check consist sorting order
#samtools view /projects/bioinfo/holger/projects/krause_chipseq/alignments_mmfilt/H3HA_Sphere_mmf.bam  | cut -f 3 | uniq
#sort -k1,1g -k2,2n $regionModel.bed | cut -f1 | uniq
### --> use reorder sam http://sourceforge.net/p/samtools/mailman/samtools-help/thread/4DB6F0CD.6050403@umdnj.edu/

sort -k1V $bowtieIndex.fa.dict_tmp > $bowtieIndex.dict

samtools view -h $bamFile | removeMultiMappers > ${bamBaseName}_mmf.tmp.bam
samtools index ${bamBaseName}_mmf.tmp.bam # because ReorderSam runs substantially faster if the input is an indexed BAM file.

# Use Picard ResortSam for resorting --> this will sort according to chromosome order in reference fasta only
picard ReorderSam I=${bamBaseName}_mmf.tmp.bam O=${bamBaseName}_mmf.bam REFERENCE=$bowtieIndex.fa
##    samtools view -h $bamFile | removeMultiMappers | picard ReorderSam I=/dev/stdin O=${bamBaseName}_mmf.bam REFERENCE=$bowtieIndex.fa
samtools index ${bamBaseName}_mmf.bam




## extract chromosome from bam file
samtools view -bo test.bam /home/brandl/mnt/chip-seq_study/ChIPSeq_February_2014/alignments_trimmed_nomulti_pooled/H2Az.bam 1



########################################################################################################################
## macs2 playground

macs2 -n --broad --gsize
#macs2 -n --gsize


########################################################################################################################
### deeptools

## https://github.com/fidelram/deepTools/wiki/All-command-line-options#bamCorrelate
#bamCorrelate bins --bamfiles $bamFiles --region 10:1:100000 --plotFile="bam_correlation.png" --numberOfProcessors=4 --corMethod spearman

## see how well bam files correlate using untrimmed data
bamCorrelate bins --bamfiles $bamFiles --plotFile="bam_correlation_untrimmed.png" --numberOfProcessors=4 --corMethod spearman

########################################################################################################################
### bam-index all files in directory


#for bamFile in $(ls $baseDir/alignments_trimmed_nomulti_pooled/*.bam); do (samtools index $bamFile ) & done


########################################################################################################################
### Peak calling with SPP

Rscript /home/brandl/bin/phantompeakqualtools/run_spp.R -c="../alignments_untrimmed/H2Az_Rep1_Lane2_Lib4454.0001.bam" -savp -out="test.txt"




########################################################################################################################
### other qc

## http://www.nature.com/nmeth/journal/v11/n1/full/nmeth.2786.html
# Teytelman et al. discovered that highly expressed loci were always enriched in ChIP peaks, regardless of which protein was pulled dow


#########################################################################################################################
#### CHANCE


mcdir $baseDir/qc_chance

## convert example to sam format
bamToBed -i $baseDir/alignments_trimmed/H2Az_Rep1_Lane1_Lib4454_ca.bam > H2Az_Rep1_Lane1_Lib4454_ca.bed
samtools view -h -o H2Az_Rep1_Lane1_Lib4454_ca.sam  $baseDir/alignments_trimmed/H2Az_Rep1_Lane1_Lib4454_ca.bam

/local/home/henry/bin/CHANCE/run_chance_linux.sh /usr/local/MATLAB/MATLAB_Compiler_Runtime/v717/


########################################################################################################################
## Use phantompeakqualtools to calculate quality tag (strand cross-correlation of peak)

# did the antibody-treatment enrich sufficiently so that the ChIP signal can be separated from the background signal?
## -->  use https://code.google.com/p/phantompeakqualtools/

mcdir $baseDir/qc_phantompeaks

#Rscript /local/home/brandl/bin/phantompeakqualtools/run_spp.R
#Rscript run_spp.R <bamfile> -savp -out=<outfile>

ctrlBam=$baseDir/alignments_untrimmed/H3-3_Rep1_Lane1_Lib4453.bam
#DEBUG ctrlBam=$baseDir/alignments_untrimmed/H3K4me3_Rep2_Lane2_Lib4455.bam


Rscript /projects/bioinfo/holger/bin/phantompeakqualtools/run_spp.R
for bamFile in $baseDir/alignments_untrimmed/*bam ; do
    echo "processing $bamFile..."
#    Rscript /home/brandl/bin/phantompeakqualtools/run_spp.R -c=$bamFile -savp -ou  t=$(basename $bamFile).pt.log
    ## one output file is enough  because output is tabular including inpuyt file name
#    Rscript /home/brandl/bin/phantompeakqualtools/run_spp.R -c=$bamFile -i=$ctrlBam -savp -savd -odir=$(pwd) -out=phantom_qc.txt

    ## with control
#    ( Rscript /home/brandl/bin/phantompeakqualtools/run_spp.R -c=$bamFile -savp -savd -odir=$(pwd) -out=phantom_qc_noctrl.txt &> $(basename $bamFile).pt.log ) &

    ## without control
#    ( Rscript /home/brandl/bin/phantompeakqualtools/run_spp.R -c=$bamFile -savp -savd -odir=$(pwd) -out=phantom_qc_noctrl.txt &> $(basename $bamFile).pt.log ) &
done

wait

## todo visualize results
#phantom output format
#format:Filename<tab>numReads<tab>estFragLen<tab>corr_estFragLen<tab>PhantomPeak<tab>corr_phantomPeak<tab>argmin_corr<tab>min_corr<tab>Normalized SCC (NSC)<tab>Relative SCC (RSC)<tab>QualityTag)


mailme "phantom qc done"


########################################################################################################################
### sorted counting
#


## perform the counts analysis (see https://www.biostars.org/p/13980/)

## presort the bed files for faster counting
for bedFile in $(ls *bed | grep -v sortedbam | grep -v sorted | grep -v pooled); do
    sort -k1,1 -k2,2n $bedFile > ${bedFile%%.bed}.sorted.bed
done

ll $bamFiles

## convert the bams to sorted bed for fast counting
for bamFile in $bamFiles; do
    # DEBUG bamFile=/projects/bioinfo/holger/projects/khan_chipsseq_h1/alignments_mmfilt/H1M_Dome_mmf.bam
    bamBaseName=$(basename $(basename $bamFile .bam) _mmf)
    mysub "${project}__ib__${bamBaseName}" " bamToBed -i $bamFile | sort -k1,1 -k2,2n  > ${bamBaseName}.sortedbam.bed" | joblist .bam2bed
done
wait4jobs .bam2bed


for bamBedFile in $(ls *sortedbam.bed); do
    for sortedBed in $(ls *sorted.bed); do
#    for sortedBed in zv9_ens79_promoters1kb.sorted.bed; do
#    for sortedBed in zv9_ens79_firstex.sorted.bed zv9_ens79_lastex.sorted.bed; do
        # DEBUG bamBedFile=H3K4_Shield.sortedbam.bed
        # DEBUG sortedBed=zv9_ens79_genebodies.sorted.bed

(
        bamBaseName=$(basename $bamBedFile .sortedbam.bed)
        bedBaseName=$(basename $sortedBed .sorted.bed | sed 's/zv9_ens79_//g')

        ## note bam-mode not yet supported for sorted
#        mysub "${project}__ib__${bamBaseName}" "
            intersectBed -c -sorted -a $sortedBed -b $bamBedFile > ${bamBaseName}.${bedBaseName}.region_counts.txt
#        " | joblist .counts

        # bamToBed -i $bamFile | sort -k1,1 -k2,2n | intersectBed -c -sorted -a $sortedBed -b stdin > ${bamBaseName}.${bedBaseName}.region_counts.txt
        # bamToBed -i $bamFile | sort -k1,1 -k2,2n  > ${bamBaseName}.sorted.bed
        # intersectBed -c -sorted -a $regionModel.sorted.bed -b ${bamBaseName}.sorted.bed > ${bamBaseName}.region_counts.txt
)&
    done
done
wait
#wait4jobs .counts
mailme "${project} counts analysis done"


########################################################################################################################
### motifs

## run meme-chip

########################################################################################################################
### profiles

## try deepTools

## differential binding diffbind


diffReps
"Negative Binomial tests implemented in diffReps (Shen et al. 2013) were used to detect differential histone
modification regions using a sliding window of 600bp (H3K4me3) or 5000bp (H3K27me3), p-value cutoff 1e-
6, sharp peaks mode for H3K4me3 (--nsb 20) and broad peak mode for H3K27me3 (--nsb 2), hg19 as reference
genome, and an average fragment size of 250bp (rest of parameters default). Differential histone modifications
regions not overlapping (at least 1bp) significant chromatin marks previously detected during peak calling at
least in one of conditions under comparison were removed. Regions were ranked by their fold-change (FC), and
reported as differentially enriched only if the absolute FC≥1.5, and Benjamini-Hochberg corrected p-value≤1e-3.
As described on the figures, figure legends and results, for Fig. S6C and S6D only the thresholds for
statistical significance were relaxed as it follows: binomial test p-value cutoff 1e-4 (diffReps standard); absolute
FC≥1.25, and Benjamini-Hochberg corrected p-value≤1e-2 (FDR 1%)."
